﻿using Performance.EntityModels;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Performance.Repository
{
    public partial class PerforReportRepository : PerforRepository<PerReport>
    {
        public PerforReportRepository(PerformanceDbContext context) : base(context)
        {
        }

        /// <summary>
        /// 月群体人均绩效
        /// </summary>
        /// <returns></returns>
        public List<PerReport> GetAvgPerfor(int hospitalid)
        {

            string sql = @"select concat(allot.year,'-',lpad(allot.month,2,'0')) x, positionname y,round(avgvalue,2) value
                from res_baiscnorm bc left join per_allot allot on bc.allotid = allot.id where allot.hospitalid = @hospitalid 
                order by str_to_date(concat(allot.month, '/', allot.year),'%m/%Y')";
            return DapperQuery(sql, new { hospitalid }).ToList();
        }

        /// <summary>
        /// 人群绩效比
        /// </summary>
        /// <param name="hospitalid"></param>
        /// <returns></returns>
        public List<PerReport> AvgRatio(int hospitalid)
        {
            string sql = @"select concat(allot.year,'-',lpad(allot.month,2,'0')) x, bc.PositionName y,round(bc.AvgValue / rbn.AvgValue,2) value
                from res_baiscnorm bc left join res_baiscnorm rbn on bc.allotid = rbn.allotid and rbn.positionname = '临床科室主任人均绩效'
                left join per_allot allot on bc.allotid = allot.id where allot.hospitalid = @hospitalid 
                order by str_to_date(concat(allot.month, '/', allot.year),'%m/%Y');";
            return DapperQuery(sql, new { hospitalid }).ToList();
        }

        /// <summary>
        /// 门诊患者均次费用
        /// </summary>
        /// <returns></returns>
        public List<PerReport> OutFeeAvg(int hospitalId, List<string> date)
        {
            string sql = @"select concat(year,'-',lpad(month,2,'0')) y,deptname x,round(sum(fee) / sum(persontime),2) value from hos_personfee where source = '门诊' and concat(year,'-',lpad(month,2,'0')) and hospitalid = @hospitalId in @date group by year,month,deptname order by y asc,value desc;";
            return DapperQuery(sql, new { date, hospitalId }).ToList();
        }

        /// <summary>
        /// 住院患者均次费用
        /// </summary>
        /// <returns></returns>
        public List<PerReport> InpatFeeAvg(int hospitalId, List<string> date)
        {
            string sql = @"select concat(year,'-',lpad(month,2,'0')) y,deptname x,round(sum(fee) / sum(persontime),2) value from hos_personfee where source = '住院' and concat(year,'-',lpad(month,2,'0')) in @date and hospitalid = @hospitalId group by year,month,deptname order by y asc,value desc;";
            return DapperQuery(sql, new { date, hospitalId }).ToList();
        }

        /// <summary>
        /// 科室药占比
        /// </summary>
        /// <returns></returns>
        public List<PerReport> Medicine(int hospitalId, List<string> date)
        {
            string sql = @"select accountingunit x,concat(year,'-',lpad(month,2,'0')) y,round((sum(if(cd.id is null,0,cellvalue))  / sum(cellvalue))*100,2) value 
                from per_allot aot join per_sheet sht on aot.id=sht.allotid join im_data dt on dt.sheetid=sht.id 
				left join cof_drugtype cd on cd.allotid=dt.allotid and cd.charge=dt.TypeName and cd.chargetype in ('药品') where unittype=1 and sheettype=3 
                and sheetname like '%就诊收入' and ifnull(accountingunit,'') not in ('') and concat(year,'-',lpad(month,2,'0')) 
                in @date and hospitalid = @hospitalId group by year,month,accountingunit order by y asc,value desc;";
            return DapperQuery(sql, new { hospitalId, date }).ToList();
        }

        /// <summary>
        /// 科室有效收入占比
        /// </summary>
        /// <returns></returns>
        public List<PerReport> Income(int hospitalId, List<string> date)
        {
            string sql = @"select accountingunit x,concat(year,'-',lpad(month,2,'0')) y,round((sum(if(cd.id is null,cellvalue,0)) / sum(cellvalue))*100,2) value
                from per_allot aot join per_sheet sht on aot.id=sht.allotid join im_data dt on dt.sheetid=sht.id 
				left join cof_drugtype cd on cd.allotid=dt.allotid and cd.charge=dt.TypeName and cd.chargetype in ('药品','耗材') 
				where unittype=1 and sheettype=3 and sheetname like '%就诊收入' and ifnull(accountingunit,'') not in ('') and concat(year,'-',lpad(month,2,'0')) 
                in @date and hospitalid = @hospitalId group by year,month,accountingunit order by y asc,value desc;";
            return DapperQuery(sql, new { hospitalId, date }).ToList();
        }

        #region 首页报表
        /// <summary>
        /// 绩效发放金额
        /// </summary>
        /// <param name="currentDate"></param>
        /// <param name="yoyDate"></param>
        /// <param name="chainDate"></param>
        /// <returns></returns>
        public List<PerReport> PerforPayment(int hospitalId, string currentDate, string yoyDate, string chainDate)
        {
            string sql = $"select t1.*,ifnull(t2.RealGiveFee, 0.0000) value from (select '{currentDate}' x,'当月发放金额' y union select '{yoyDate}' x,'同期发放金额' y union select '{chainDate}' x,'环比发放金额' y)t1 left join report_allot_summary t2 on t1.x = concat(t2.year,'-',lpad(t2.month,2,'0')) and t2.HospitalId = @hospitalId;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 绩效发放金额占全院收入占比
        /// </summary>
        /// <param name="currentDate"></param>
        /// <param name="yoyDate"></param>
        /// <param name="chainDate"></param>
        /// <returns></returns>
        public List<PerReport> IndexPerforRatio(int hospitalId, string currentDate, string yoyDate, string chainDate)
        {
            string sql = $"select t1.*, round(ifnull(t2.RealGiveFee, 0.0000)/t3.income*100, 2)value from(select '{currentDate}' x,'当月发放占比' y union select '{yoyDate}' x,'同期发放占比' y union select '{chainDate}' x,'环比发放占比' y)t1 left join report_allot_summary t2 on t1.x = concat(t2.year,'-', lpad(t2.month,2,'0')) and t2.HospitalId = @hospitalId left join (select year, month, sum(CellValue) income from report_original_income where HospitalId = @hospitalId group by year, month)t3 on t1.x = concat(t3.year,'-', lpad(t3.month,2,'0'));";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 首页药占比（本月、环比、同期）
        /// </summary>
        /// <param name="hospitalId">医院Id</param>
        /// <param name="currentDate">本月日期</param>
        /// <param name="yoyDate">同比日期</param>
        /// <param name="chainDate">环比日期</param>
        /// <returns></returns>
        public List<PerReport> IndexDrugRatio(int hospitalId, string currentDate, string yoyDate, string chainDate)
        {
            string sql = $"select t1.*,round(t2.CellValue/t3.`value`*100, 2) value from (select '{currentDate}' x,'当月药占比' y union select '{yoyDate}' x,'同期药占比' y union select '{chainDate}' x,'环比药占比' y)t1 left join (select concat(year,'-',lpad(month,2,'0')) date,sum(CellValue) CellValue from report_original_income t1 inner join cof_drugtype t2 on t1.TypeName = t2.Charge and t1.AllotID = t2.AllotId and t2.ChargeType = '药品' where HospitalID = @hospitalId group by year,month) t2 on t1.x = t2.date left join (select concat(year,'-',lpad(month,2,'0')) date,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year,month)t3 on t1.x = t3.date;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 首页材料占比（本月、环比、同期）
        /// </summary>
        /// <param name="hospitalId">医院Id</param>
        /// <param name="currentDate">本月日期</param>
        /// <param name="yoyDate">同比日期</param>
        /// <param name="chainDate">环比日期</param>
        /// <returns></returns>
        public List<PerReport> IndexMaterialRatio(int hospitalId, string currentDate, string yoyDate, string chainDate)
        {
            string sql = $"select t1.*,round(t2.CellValue/t3.`value`*100, 2) value from (select '{currentDate}' x,'当月材料占比' y union select '{yoyDate}' x,'同期材料占比' y union select '{chainDate}' x,'环比材料占比' y)t1 left join (select concat(year,'-',lpad(month,2,'0')) date,sum(CellValue) CellValue from report_original_income t1 inner join cof_drugtype t2 on t1.TypeName = t2.Charge and t1.AllotID = t2.AllotId and t2.ChargeType = '耗材' where HospitalID = @hospitalId group by year,month) t2 on t1.x = t2.date left join (select concat(year,'-',lpad(month,2,'0')) date,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year,month)t3 on t1.x = t3.date;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 首页结构占比
        /// </summary>
        /// <returns></returns>
        public List<PerReport> IndexStructRatio(int hospitalId, string currentDate)
        {
            string sql = $"select TypeName y,t1.y x,round(CellValue/t2.`value`*100,2) value from (select '{currentDate}' y,TypeName,sum(CellValue) CellValue from report_original_income where HospitalID = @hospitalId and concat(year,'-',lpad(month,2,'0')) = '{currentDate}' group by TypeName) t1 inner join (select '{currentDate}' y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId and concat(year,'-',lpad(month,2,'0')) = '{currentDate}')t2 on t1.y = t2.y order by value desc;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }
        #endregion

        #region 菜单报表
        /// <summary>
        /// 业务总收入
        /// </summary>
        /// <returns></returns>
        public List<PerReport> GeneralIncome(int hospitalId, int isOnlyYear)
        {
            string sql = $"select year x,'业务总收入（年）' y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year;";
            if (isOnlyYear != 1)
                sql = $"select concat(year,'-',lpad(month,2,'0')) x,'业务总收入（月）' y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year,month;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 门诊、住院业务收入占比
        /// </summary>
        /// <returns></returns>
        public List<PerReport> InHosIncome(int hospitalId, int isOnlyYear, int isInHos)
        {
            string xAxis = isInHos == 1 ? "住院" : "门诊";
            string formula = isInHos == 1 ? "t.InHos/(t.InHos+t.Outpatient)" : "t.Outpatient/(t.InHos+t.Outpatient)";

            string sql = $"select '{xAxis}' x,year y,round({formula} * 100, 2) value from (select year,sum(case SourceType when '住院' then CellValue else 0 end) InHos,sum(case SourceType when '门诊' then CellValue else 0 end) Outpatient from report_original_income where hospitalid = @hospitalId group by year)t;";
            if (isOnlyYear != 1)
                sql = $"select '{xAxis}' x,concat(year,'-',lpad(month,2,'0')) y,round({formula} * 100, 2) value from (select year,month,sum(case SourceType when '住院' then CellValue else 0 end) InHos,sum(case SourceType when '门诊' then CellValue else 0 end) Outpatient from report_original_income where hospitalid = @hospitalId group by year,month)t;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 业务收入结构占比
        /// </summary>
        /// <returns></returns>
        public List<PerReport> StructRatio(int hospitalId, int isOnlyYear)
        {
            string sql = $"select TypeName x,t1.y,round(CellValue/t2.`value`*100,2) value from (select year y,TypeName,sum(CellValue) CellValue from report_original_income where HospitalID = @hospitalId group by year,TypeName) t1 inner join (select year y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year)t2 on t1.y = t2.y order by y asc,value desc;";
            if (isOnlyYear != 1)
                sql = $"select TypeName x,t1.y,round(CellValue/t2.`value`*100,2) value from (select concat(year,'-',lpad(month,2,'0')) y,TypeName,sum(CellValue) CellValue from report_original_income where HospitalID = @hospitalId group by year,month,TypeName) t1 inner join (select concat(year,'-',lpad(month,2,'0')) y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year,month)t2 on t1.y = t2.y order by y asc,value desc;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 药占比
        /// </summary>
        /// <param name="hospitalId"></param>
        /// <param name="isOnlyYear"></param>
        /// <returns></returns>
        public List<PerReport> DrugRatio(int hospitalId, int isOnlyYear)
        {
            string sql = $"select TypeName x,t1.y,round(CellValue/t2.`value`*100,2) value from (select year y,TypeName,sum(CellValue) CellValue from report_original_income t1 inner join cof_drugtype t2 on t1.TypeName = t2.Charge and t1.AllotID = t2.AllotId and t2.ChargeType = '药品' where HospitalID = @hospitalId group by year,TypeName) t1 inner join (select year y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year)t2 on t1.y = t2.y;";
            if (isOnlyYear != 1)
                sql = $"select TypeName x,t1.y,round(CellValue/t2.`value`*100,2) value from (select concat(year,'-',lpad(month,2,'0')) y,TypeName,sum(CellValue) CellValue from report_original_income t1 inner join cof_drugtype t2 on t1.TypeName = t2.Charge and t1.AllotID = t2.AllotId and t2.ChargeType = '药品' where HospitalID = @hospitalId group by year,month,TypeName) t1 inner join (select concat(year,'-',lpad(month,2,'0')) y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year,month)t2 on t1.y = t2.y;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 材料占比
        /// </summary>
        /// <param name="hospitalId"></param>
        /// <param name="isOnlyYear"></param>
        /// <returns></returns>
        public List<PerReport> MaterialRatio(int hospitalId, int isOnlyYear)
        {
            string sql = $"select TypeName x,t1.y,round(CellValue/t2.`value`*100,2) value from (select year y,TypeName,sum(CellValue) CellValue from report_original_income t1 inner join cof_drugtype t2 on t1.TypeName = t2.Charge and t1.AllotID = t2.AllotId and t2.ChargeType = '耗材' where HospitalID = @hospitalId group by year,TypeName) t1 inner join (select year y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year)t2 on t1.y = t2.y;";
            if (isOnlyYear != 1)
                sql = $"select TypeName x,t1.y,round(CellValue/t2.`value`*100,2) value from (select concat(year,'-',lpad(month,2,'0')) y,TypeName,sum(CellValue) CellValue from report_original_income t1 inner join cof_drugtype t2 on t1.TypeName = t2.Charge and t1.AllotID = t2.AllotId and t2.ChargeType = '耗材' where HospitalID = @hospitalId group by year,month,TypeName) t1 inner join (select concat(year,'-',lpad(month,2,'0')) y,sum(CellValue) value from report_original_income where hospitalid = @hospitalId group by year,month)t2 on t1.y = t2.y;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 绩效发放金额占全院收入占比
        /// </summary>
        /// <param name="hospitalId"></param>
        /// <param name="isOnlyYear"></param>
        /// <returns></returns>
        public List<PerReport> PerforRatio(int hospitalId, int isOnlyYear)
        {
            string sql = $"select t2.x,'绩效发放金额占全院收入占比（年）' y,round(t1.RealGiveFee/t2.income * 100, 2) value from (select `year`,sum(realgivefee) realgivefee from report_allot_summary where HospitalID = @hospitalId group by year) t1 inner join (select year x,sum(CellValue) income from report_original_income where HospitalId = @hospitalId group by year)t2 on t1.`Year` = t2.x;";
            if (isOnlyYear != 1)
                sql = $"select t2.x,'绩效发放金额占全院收入占比（月）' y,round(t1.RealGiveFee/t2.income * 100, 2) value from report_allot_summary t1 inner join (select concat(year,'-',lpad(month,2,'0')) x,sum(CellValue) income from report_original_income where HospitalId = @hospitalId group by year,month)t2 on concat(t1.year,'-',lpad(t1.month,2,'0')) = t2.x where t1.HospitalID = @hospitalId;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 绩效群体收入
        /// </summary>
        /// <param name="hospitalId"></param>
        /// <param name="isOnlyYear"></param>
        /// <returns></returns>
        public List<PerReport> PerforGroup(int hospitalId, int isOnlyYear)
        {
            string sql = $"select PositionName x,year y,round(sum(avgvalue), 2) value from (select t1.PositionName,year,AvgValue from res_baiscnorm t1 inner join per_allot t2 on t1.AllotID = t2.Id and t2.HospitalId = @hospitalId and locate('保底', t1.PositionName) = 0)t group by PositionName,year;";
            if (isOnlyYear != 1)
                sql = $"select t1.PositionName x, concat(t2.year, '-', lpad(t2.month, 2, '0')) y,round(AvgValue, 2) value from res_baiscnorm t1 inner join per_allot t2 on t1.AllotID = t2.Id and t2.HospitalId = @hospitalId and locate('保底', t1.PositionName) = 0;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 医生核算单元人均绩效
        /// </summary>
        /// <param name="hospitalId"></param>
        /// <param name="isOnlyYear"></param>
        /// <returns></returns>
        public List<PerReport> DoctorAvg(int hospitalId, int isOnlyYear)
        {
            string sql = $"select `Year` y,AccountingUnit x,round(sum(avg),2) value from (select t1.AccountingUnit,t1.Avg,t2.`Year`,t2.`Month`,t2.ID from res_account t1 inner join per_allot t2 on t1.AllotID = t2.ID and t2.HospitalId = @hospitalId and t1.UnitType != 2)t group by year,AccountingUnit;";
            if (isOnlyYear != 1)
                sql = $"select concat(year,'-',lpad(month,2,'0')) y,AccountingUnit x,round(sum(avg),2) value from (select t1.AccountingUnit,t1.Avg,t2.`Year`,t2.`Month`,t2.ID from res_account t1 inner join per_allot t2 on t1.AllotID = t2.ID and t2.HospitalId = @hospitalId and t1.UnitType != 2)t group by year,month,AccountingUnit;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }

        /// <summary>
        /// 护理核算单元人均绩效
        /// </summary>
        /// <param name="hospitalId"></param>
        /// <param name="isOnlyYear"></param>
        /// <returns></returns>
        public List<PerReport> NurseAvg(int hospitalId, int isOnlyYear)
        {
            string sql = $"select `Year` y,AccountingUnit x,round(sum(avg),2) value from (select t1.AccountingUnit,t1.Avg,t2.`Year`,t2.`Month`,t2.ID from res_account t1 inner join per_allot t2 on t1.AllotID = t2.ID and t2.HospitalId = @hospitalId and t1.UnitType = 2)t group by year,AccountingUnit;";
            if (isOnlyYear != 1)
                sql = $"select concat(year,'-',lpad(month,2,'0')) y,AccountingUnit x,round(sum(avg),2) value from (select t1.AccountingUnit,t1.Avg,t2.`Year`,t2.`Month`,t2.ID from res_account t1 inner join per_allot t2 on t1.AllotID = t2.ID and t2.HospitalId = @hospitalId and t1.UnitType = 2)t group by year,month,AccountingUnit;";
            return DapperQuery(sql, new { hospitalId }).ToList();
        }
        #endregion
    }
}
